use raas_types::raas;
use std::collections::HashMap;
use thiserror::Error;
use tokio::io::DuplexStream;

pub mod connection_handler;
pub mod robot_connector;
pub mod robot_manager;

pub const ROBOT_MESSAGE_QUEUE_SIZE: usize = 10;
const ROBOT_TIMEOUT: std::time::Duration = std::time::Duration::from_secs(60);

#[derive(Debug)]
pub struct ConnectorHandle {
    pub stream: DuplexStream,
    pub tags: Vec<raas::register::BotTypes>,
}

#[derive(Debug)]
pub struct ConnectorManager {
    pub next_id: u32,
    pub connectors: HashMap<u32, ConnectorHandle>,
}

#[derive(Error, Debug)]
pub enum Error {
    #[error("IO Error")]
    Io(#[from] std::io::Error),
    #[error("Protobuf Encode Error")]
    ProtobufEncode(#[from] prost::EncodeError),
    #[error("Protobuf Decode Error")]
    ProtobufDecode(#[from] prost::DecodeError),
    #[error("Connection to robot has been closed")]
    ConnectionClosed(u32),
    #[error("No robots to handle requests")]
    NoRobotsToHandleRequest,
    #[error("Timed out waiting for response")]
    Timeout(#[from] tokio::time::error::Elapsed),
}
