use log::info;
use prost::Message;
use raas_types::raas::service::raas_server::Raas;
use raas_types::raas::{recv_raas_msg, send_raas_msg};
use std::ops::DerefMut;
use tokio::io::DuplexStream;
use tonic::{Request, Response, Status};

pub struct RaaSServer {
    command_stream: tokio::sync::Mutex<DuplexStream>,
}

impl RaaSServer {
    pub fn new(command_stream: DuplexStream) -> Self {
        Self {
            command_stream: tokio::sync::Mutex::new(command_stream),
        }
    }
}

#[tonic::async_trait]
impl Raas for RaaSServer {
    async fn send_request(
        &self,
        request: Request<raas_types::raas::cmd::Request>,
    ) -> Result<Response<raas_types::raas::resp::Response>, Status> {
        let mut command_stream = self.command_stream.lock().await;

        let remote_addr = request.remote_addr().unwrap();
        let request = request.into_inner();
        info!("Got request from {}: {:?}", remote_addr, request);

        let mut msg = Vec::new();

        request.encode(&mut msg).unwrap();

        send_raas_msg(command_stream.deref_mut(), msg)
            .await
            .unwrap();

        let resp = recv_raas_msg(command_stream.deref_mut()).await.unwrap();

        let resp = raas_types::raas::resp::Response::decode(&*resp.msg).unwrap();

        Ok(Response::new(resp))
    }
}
